# coding: utf-8
try:
    import cgitb
    import json
    import logging
    import sys
    
    from ws4py.server.handler.tornadohandler import WebSocketHandler
    
    from sharedvariables import agents_list
    from dbinterface import agent_login, save_log, set_installation_status, set_agent_status
    from servertoagent import auth_request, auth_response, exit_request
    from brousershandler import notify_browsers
except Exception as e:
    logging.error(e)
    logging.error(cgitb.text(sys.exc_info()))

class AgentsHandler(WebSocketHandler):
    def open(self):
        self._agent_name = ""
        self._logined = False
        self._no = 0
        self._status = 'Offline'
        self._recent_notify = None
    
    def reset_counter(self):
        self._no = 0
        
    @property
    def status(self):
        return self._status
    
    def to_exit(self):
        """ Обработка запроса на отключение от сервера
        """
        try:
            self.write_message(exit_request())
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))
        
    def on_close(self): 
        """ Событие при отключении агента
        """
        try:
            self._set_status('Offline')           
            agents_list.pop(self._agent_name)
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))
        
    def on_message(self, message):
        """ Обработка сообщения от сервера
        """
        try:
            message = json.loads(message)
            command = message['command']
                
            if not self._logined and command != 'auth_response':
                self.write_message(auth_request())
            
            if command == 'auth_response':
                self._auth(message['login'], message['password'])
            elif command == 'status_response':
                self._set_status(message['status'])
            elif command == 'change_installation_status':
                set_installation_status(self._agent_name, message['status'])
            elif command == 'log_message':
                self._save_log( message['msg_type'], message['msg'])
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))
                
    def _auth(self, login, password):
        """ Обработка запроса на авторизацию
        """
        try:
            self._logined = True
            agents_list[login] = self
            self._agent_name = login
            agent_login(login)
            self.write_message(auth_response(True))
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))

    def _set_status(self, status):
        """ Обработка запроса на изменение статуса агента
        """
        try:
            #need_update_browsers = (self._status == 'Online' and status == 'Offline') or \
            #                       (self._status == 'Offline' and status == 'Online')
            need_update_browsers = True
            self._status = status        
            logging.info('Agent' + str(self._agent_name) + 'has new status: ' + str(status))
            set_agent_status(self._agent_name, status)
            if need_update_browsers:
                notify_browsers()
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))

    def _save_log(self, msg_type, msg):
        """ Обработка нового сообщения, части лога, которое поступает при инсталляции
        """
        try:
            self._no += 1
            save_log(self._agent_name, msg_type, msg, self._no)
            notify_browsers(msg, msg_type)
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))