# coding: utf-8

try:
    import os
    import cgitb
    from threading import Thread
    import time
    import logging
    import sys
    
    import tornado.ioloop
    import tornado.web
    import tornado.httpserver
    
    from agentshandler import AgentsHandler
    from brousershandler import BrowsersHandler
    from sharedvariables import agents_list
    from sockjs.tornado import SockJSRouter
    import settings
except Exception as e:
    logging.error(e)
    logging.error(cgitb.text(sys.exc_info()))

AGENTS_SOCKET_PORT = 8001
BROWSERS_SOCKET_PORT = 8002
STATIC_PATH = os.path.join(os.path.dirname(__file__), 'static')

class AgentsServerThread(Thread):
    """ Поток в котором крутится цикл, взаимодействующий с агентами
    """
    def __init__(self, urls, port, *args, **kwargs):
        try:
            Thread.__init__(self, *args, **kwargs)
            app = tornado.web.Application(urls)
            http_server = tornado.httpserver.HTTPServer(app)
            http_server.listen(port)
            self.daemon = True
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))
    
    def run(self):
        try:
            self._loop = tornado.ioloop.IOLoop.instance()
            self._loop.start()
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))
            
class SockjsServerThread(Thread):
    """ Поток в котором крутится цикл, взаимодействующий с браузерами 
    """
    def __init__(self, handler, url, port, *args, **kwargs):
        try:
            Thread.__init__(self, *args, **kwargs)
            EchoRouter = SockJSRouter(handler, url)
            app = tornado.web.Application(EchoRouter.urls)
            app.listen(port)
            self.daemon = True
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))
    
    def run(self):
        try:
            self._loop = tornado.ioloop.IOLoop.instance()
            self._loop.start()
        except Exception as e:
            logging.error(e)
            logging.error(cgitb.text(sys.exc_info()))
    
def ensure_dir(d):
    if not os.path.exists(d):
        os.makedirs(d)
    
def start_server_agents():
    """ Запуск компонента сервера, который общается с агентами c помощью ws4py
    """
    ensure_dir(settings.RECIPES_PATH)
    agents_urls = [(r'/agent-server', AgentsHandler), ]
    agents_server_thread = AgentsServerThread(agents_urls, AGENTS_SOCKET_PORT)
    agents_server_thread.start()
    
def start_server_browsers():
    """ Запуск компонента сервера, который общается с браузерами с помощью Sockjs
    """
    agents_server_thread = SockjsServerThread(BrowsersHandler, '/socket', BROWSERS_SOCKET_PORT)
    agents_server_thread.start()
    
def start_server():
    """ Запуск компонент
    """
    start_server_agents()
    start_server_browsers()
        
if __name__ == '__main__':
    start_server()
