# -*- coding: utf-8 -*-

try:
    import cgitb
    import sys
    import logging
    
    from django.core.context_processors import csrf
    from django.contrib.auth.views import login, logout
    from django.contrib import auth
    from django.http import HttpResponseRedirect
    from django.shortcuts import render_to_response
    from django.contrib.auth.decorators import login_required
    
    import backend.sharedvariables as var
    from backend.dbinterface import get_installations, get_installations_by_time, rename_agent, delete_agent, \
                        get_recent_installation_log, new_installation, get_agent_name, get_agent_status, \
                        is_recipe_exists, save_recipe, get_recipe_id, get_recipe_name, get_agents_list
    from backend.servertoagent import installation_request, exit_request
    from backend.server import start_server
    from backend.recipes import get_recipes, get_recipe_template, get_next_version, \
                        not_valid_error_message, create_recipe_path_if_not_exists, recipe_save, get_recipe_content, \
                        get_last_version, get_default_config, delete_recipe_version, recipe_delete, get_version_count
    import junta_xml
    
except Exception as e:
    logging.error(e)
    logging.error(cgitb.text(sys.exc_info()))


def setup_logger(log_file, rotation=5*1024, count=5):
    """ Настраивает логирование в STDOUT и указанный лог-файл @a log_file.
        Объём ротации задаётся 
    """
    formatter = logging.Formatter('[%(asctime)s][%(levelname)s][%(module)s]: %(message)s', datefmt='%H:%M:%S %d-%m-%Y')
    
    handler = logging.handlers.RotatingFileHandler(log_file, maxBytes=rotation*1024, backupCount=count)
    handler.setFormatter(formatter)
    logging.getLogger().addHandler(handler)
    
    console = logging.StreamHandler()
    console.setFormatter(formatter)
    logging.getLogger().addHandler(console)
    logging.getLogger().setLevel(logging.DEBUG)
    logging.getLogger('django.db.backends').setLevel(logging.ERROR) 

def server_run():          
    """ Запуск сервера, который связывается с агентами, если он не запущен. 
    """ 
    if var.server_state != 'started': 
        try:
            setup_logger('/var/log/junta-webserver.log')
        except IOError:
            print >>sys.stderr, 'Could not setup logging to file'
        start_server()
        var.server_state = 'started'
        logging.info('Server started.')


def get_common_context(request):
    """ Получение общего контента для всех страниц.
    """ 
    try:
        server_run()
        c = {}
        c.update(csrf(request))
        c['user'] = request.user
        c['recipes'] = get_recipes()
        c['servers'] = get_agents_list()
        c['server_is_started'] = (var.server_state == 'started')
        return c
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))

def get_server_context(server_id):
    """ Получение контента для определенного сервера
    """ 
    try:
        c = {}
        c['server_id'] = server_id
        c['server_name'] = get_agent_name(server_id)
        c['status'] = get_agent_status(server_id)
        c['installations'] = get_installations(server_id)
        c['installations_by_time'] = get_installations_by_time(server_id)
        return c
    except Exception as e:
        logging.error(e)

def login_user(request, params=''):
    """ Пользователь отправляет запрос на вход.
    """ 
    try:
        c = {}
        c.update(csrf(request))
        if request.method == 'POST':
            username = request.POST.get('username', '')
            password = request.POST.get('password', '')
            user = auth.authenticate(username = username, password = password)
            if user is not None:
                if user.is_active:
                    login(request, user)
                    return HttpResponseRedirect("/")
            c['err_message'] = 'Неправильный логин и/или пароль'
            return render_to_response('auth.html', c)
        else:
            return render_to_response('auth.html', c)
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")
    
def logout_user (request):
    """ Пользователь отправляет запрос на выход.
    """ 
    try:
        logout(request)
        return HttpResponseRedirect("/auth")
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")
    
@login_required
def home(request):
    """ Запуск сервера, который связывается с агентами, если он не запущен. 
    """ 
    try:
        c = get_common_context(request)
        if u'select_all' in request.GET:
            c['select_all'] = True
        return render_to_response('home.html', c)
    except Exception as e:
        logging.error(cgitb.text(sys.exc_info()))


""" *** Представления для страницы с серверами (/servers, /installation) *** """
@login_required
def servers(request, server_id):
    """ Страница с серверами
    """
    try:
        c = get_common_context(request)
        if request.method == 'GET':
            """ просто получение всей информации
            """
            c.update(get_server_context(server_id))
            return render_to_response('servers.html', c)
        else:
            """ POST запрос
            """
            action = request.POST.get('action', '')
            if action == '':
                logging.error('Empty request at servers page')
                return HttpResponseRedirect("/")
            elif action == u'Отмена':
                """ Нажата отмена, возвращаем страницу как при GET                
                """
                return HttpResponseRedirect("/servers/" + server_id)
            elif action == u'Удалить':
                """ Удаление сервера из базы данных 
                    и отправление ему запроса на отключение
                """
                delete_agent(server_id)
                to_agent = var.agents_list[server_id]
                if to_agent is not None:
                    to_agent.write_message(exit_request())
                return HttpResponseRedirect('/')
            elif action == u'Переименновать':
                """ Запрос на переименнование сервера,
                    добавление на страницу поля ввода нового имени
                """
                c.update(get_server_context(server_id))
                c['rename'] = 'Yes'
                return render_to_response('servers.html', c)
            elif action == u'Сохранить':
                """ Сохранение нового имени сервера
                """
                rename_agent(server_id, request.POST["new_name"])
                return HttpResponseRedirect("/servers/" + server_id)
            else:
                """ Вероятнее всего здесь запрос на установку
                """
                c['is_installation'] = True
                if action == u'Установить' or action == u'Переустановить':
                    return installation_begin(c, request.POST['recipe_id'], server_id)
                elif action == u'Готово':
                    return installation_ready(c, request, server_id)
                else:
                    if "server_name" in request.POST:
                        return HttpResponseRedirect("/servers/" + server_id)
                    else:
                        logging.error("Unknown request at servers page")
                        return HttpResponseRedirect('/')
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")    
    
def installation_begin(c, recipe_id, server_id):
    """ Начало установки, запрос конфигов
    """
    try:        
        c['config'] = get_default_config(recipe_id, get_last_version(recipe_id, c['recipes']))
        c['recipe_id'] = recipe_id
        c['is_installation'] = True
        c['getting_config'] = True
        c.update(get_server_context(server_id))
        return render_to_response('servers.html', c)
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")

def installation_ready(c, request, server_id):
    """ Отправка агенту запрос на установку
    """
    try:
        recipe_id = request.POST['recipe_id']
        recipe_name = get_recipe_name(recipe_id)
        recipe_version = get_last_version(recipe_id, c['recipes'])
        c.update(get_server_context(server_id))
        
        config = {}
        default_config = get_default_config(recipe_id, recipe_version)
        hidden_config_keys = []
        for d_c in default_config:
            if d_c[1] == '':
                config[d_c[0]] = d_c[1]
                if d_c[2] == 'password':
                    hidden_config_keys.append(d_c[0])
        for arg in request.POST:
            if '.' in arg:
                key = arg.split('.')[1]
                val = request.POST[arg]
                config[key] = val     
                
        to_agent = var.agents_list[server_id]
        if to_agent is not None:
            new_installation(server_id, recipe_id, recipe_version, config, hidden_config_keys)
            to_agent.reset_counter()
            to_agent.write_message(installation_request(recipe_id, recipe_version, config))
            return HttpResponseRedirect("/".join(["/installation", server_id, recipe_id, str(recipe_version)]))
        else:
            logging.error("Unable to send installation request to agent")
            return HttpResponseRedirect('/')
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")
    
@login_required
def installation(request, server_id, recipe_id, recipe_version):
    """ Инсталляция в процессе.
        Вывод страницы с серверами с диалогом, в котором обновляется лог
    """
    try:
        c = get_common_context(request)
        c['log'] = get_recent_installation_log(server_id)   
        c['is_installation'] = True
        c.update(get_server_context(server_id))
        c['status'] = 'Installation'
        return render_to_response('servers.html', c)
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")
    
@login_required
def multi_installing(request):
    """ Запрос на установку на несколько серверов.
    """
    try:
        c = get_common_context(request)
        servers = []
        if request.method == 'POST':
            action = request.POST.get('action', '')
            
            if action != u'Готово':
                for key in dict(request.POST):
                    if str(key).startswith('install_'):
                        recipe_id = key[8:]
                servers = request.POST.getlist('servers_to_install')
                c['config'] = get_default_config(recipe_id, get_last_version(recipe_id, c['recipes']))
                c['recipe_id'] = recipe_id
                c['servers_to_install'] = servers
                c['is_installation'] = True
            elif action == u'Готово':
                recipe_id = request.POST['recipe_id']
                servers = eval(request.POST['servers_to_install'])
                recipe_version = get_last_version(recipe_id, c['recipes'])
                config = {}
                default_config = get_default_config(recipe_id, recipe_version)
                hidden_config_keys = []
                for d_c in default_config:
                    if d_c[1] == '':
                        config[d_c[0]] = d_c[1]
                        if d_c[2] == 'password':
                            hidden_config_keys.append(d_c[0])
                for arg in request.POST:
                    if '.' in arg:
                        key = arg.split('.')[1]
                        val = request.POST[arg]
                        config[key] = val
                
                for s in servers:
                    to_agent = var.agents_list[s]
                    if to_agent is not None:
                        new_installation(s, recipe_id, recipe_version, config, hidden_config_keys)
                        to_agent.reset_counter()
                        to_agent.write_message(installation_request(recipe_id, recipe_version, config))
                        logging.info('Installation request sent to ' + str(s))
                    else:
                        logging.error('Error in installation request to ' + str(s))
                return HttpResponseRedirect('/')
        return render_to_response('home.html', c)
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        logging.error("error in multi installing")
        return HttpResponseRedirect('/')
    

""" *** Представления для страницы с рецептами (/recipes) *** """
@login_required
def recipes(request):
    """ Страница с рецептами, просто меню с рецептами, без поля для редактирования
    """
    try:
        c = get_common_context(request)
        return render_to_response('recipes.html', c)
    except Exception as e:
        logging.error(e)


@login_required
def recipe_edit(request, recipe_id, recipe_version):
    """ Страница с рецептами, вместе с меню для редактирования
    """
    try:
        c = get_common_context(request)
        if request.method == 'GET':
            """ шаг 1. Вывод содержимого рецепта
            """
            if recipe_id == "add_recipe":
                """ если рецепт не выбран, имя и версия по умолчанию,
                    содержимое из шаблона
                """
                c['recipe_name'] = "new_recipe"
                c['recipe_version'] = '1'
                c['file_content'] = get_recipe_template()
            else:
                """ рецепт выбран, получаем содержимое
                """
                c['recipe_name'] = get_recipe_name(recipe_id)
                c['recipe_version'] = recipe_version
                file_content = get_recipe_content(recipe_id, recipe_version)
                c['file_content'] = junta_xml.preprocessor(file_content, False)
            return render_to_response('recipes.html', c)
        
        elif request.POST['action'] == u'Сохранить':
            """ шаг 2. Сохранение изменений
            """
            file_content = request.POST["content"]
            file_content = junta_xml.preprocessor(file_content)
            err_msg = not_valid_error_message(file_content)
            recipe_name = request.POST.get('recipe_name', 'unknown')
            if err_msg is None:
                """ если документ валиден
                """
                if not is_recipe_exists(recipe_name):
                    recipe_id = save_recipe(recipe_name)
                recipe_name = get_recipe_name(recipe_id)
                recipe_version = get_next_version(recipe_id, c['recipes'])
                create_recipe_path_if_not_exists(recipe_id)
                err_msg = recipe_save(recipe_name, recipe_version, file_content, request.user.username)
                if err_msg is None:# OK
                    """ если не произошло ошибки при сохранении
                    """
                    return HttpResponseRedirect('/recipes/' + recipe_id + '/' + recipe_version)
            """ если произошла какая-то ошибка, возврат
            """
            c['err_msg'] = err_msg
            c['recipe_name'] = recipe_name
            c['recipe_version'] = recipe_version
            c['file_content'] = junta_xml.preprocessor(file_content, False)
            return render_to_response('recipes.html', c)
        
        elif request.POST['action'] == u'Назад':   
            return render_to_response('recipes.html', c)
        elif request.POST['action'] == u'Удалить':   
            if get_version_count(recipe_id, c['recipes']) <= 1: # это был последний рецепт в своем роде
                recipe_delete(recipe_id)
                return HttpResponseRedirect("/recipes")
            else:
                delete_recipe_version(recipe_id, recipe_version)
                c = get_common_context(request)
                last_version = get_last_version(recipe_id, c['recipes'])
                print "last_version", last_version
                return HttpResponseRedirect("/".join(["/recipes", recipe_id, str(last_version)]))
        else:
            logging.error('Unknown request at recipes page: ' + str(request.POST['action']))
            return HttpResponseRedirect("/")
    except Exception as e:
        logging.error(e)
        logging.error(cgitb.text(sys.exc_info()))
        return HttpResponseRedirect("/")
    
@login_required
def add_recipe(request):
    """ Запрос на добавление рецепта, перенаправление
    """
    return recipe_edit(request, 'add_recipe', '')